package com.diss.omppapp.services;

import com.diss.omppapp.entities.AnswerEntity;
import com.diss.omppapp.entities.QuestionEntity;
import com.diss.omppapp.pojo.StudentResponseWrapper;
import com.diss.omppapp.repositories.AnswerEntityRepository;
import com.diss.omppapp.repositories.QuestionEntityRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.*;

@Component
public class MinutePaperService {

    @Autowired
    private QuestionEntityRepository questionEntityRepository;

    @Autowired
    AnswerEntityRepository answerEntityRepository;

    public List<QuestionEntity> getQuestions(String username, String paperId) {
        if ((username == null || username.isEmpty()) || (paperId == null || paperId.isEmpty())) {
            return new ArrayList<>();
        }
        Optional<List<QuestionEntity>> questionEntities = questionEntityRepository.findAllByPaperIdOrderByQuestionIdAsc(parseId(paperId));
        return questionEntities.orElseGet(ArrayList::new);
    }

    public List<AnswerEntity> getAnswers(String username, String paperId) {
        if ((username == null || username.isEmpty()) || (paperId == null || paperId.isEmpty())) {
            return new ArrayList<>();
        }
        Optional<List<AnswerEntity>> answerEntities = answerEntityRepository.findAllByPaperIdOrderByAnswerIdAsc(parseId(paperId));
        return answerEntities.orElseGet(ArrayList::new);
    }

    public Map<String,String> saveDraft(StudentResponseWrapper answers) {
        if(answers.getResponses() == null) {
            return getResponse(false);
        }

        if(!save(answers,false)) {
            return getResponse(false);
        }
        return getResponse(true);
    }

    public Map<String,String> submitResponse(StudentResponseWrapper answers) {
        if(answers.getResponses() == null) {
            return getResponse(false);
        }

        if(!save(answers,true)) {
            return getResponse(false);
        }

        return getResponse(true);
    }

    private Boolean save(StudentResponseWrapper answers, Boolean isDraft) {
        try {
            answers.getResponses().forEach(r -> {
                AnswerEntity entity = new AnswerEntity();
                int existing = hasExistingAnswer(r.getUsername(),parseId(r.getPaperId()),parseId(r.getQuestionId()));

                if( existing != -1) {
                    entity.setAnswerId(existing);
                }

                entity.setAnswerText(r.getAnswerText());
                entity.setUsername(r.getUsername());
                entity.setSubmitted(isDraft);
                entity.setPaperId(parseId(r.getPaperId()));
                entity.setQuestionId(parseId(r.getQuestionId()));
                entity.setSubmissionDate(new Date());
                answerEntityRepository.save(entity);
            });
        } catch (Exception e) {
            e.printStackTrace();
            return false;
        }
        return true;
    }

    private int hasExistingAnswer(String username, int paperId, int questionId) {
        if(answerEntityRepository.existsByUsernameAndPaperIdAndQuestionId(username,paperId,questionId)) {
            return answerEntityRepository.findByUsernameAndPaperIdAndQuestionId(username,paperId,questionId).get().answerId;
        }
        return -1;
    }

    private Map<String,String> getResponse(Boolean isGood) {
        Map<String,String> response = new HashMap<>();
        if (isGood) {
            response.put("response", "true");
            return response;
        }
        response.put("response", "false");
        return response;
    }

    private int parseId(String paperId) {
        int id = 0;

        try {
            id = Integer.parseInt(paperId);
        } catch (NumberFormatException nfe) {
            return -1;
        }
        return id;
    }

}
